@@ -1,13 +1,13 @@
 ;;; org-remark-global-tracking.el --- Track files and auto-activate Org-remark -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2021-2022 Free Software Foundation, Inc.
+;; Copyright (C) 2021-2023 Free Software Foundation, Inc.
 
 ;; Author: Noboru Ota <me@nobiot.com>
 ;; URL: https://github.com/nobiot/org-remark
 ;; Created: 15 August 2021
-;; Last modified: 13 February 2022
+;; Last modified: 15 April 2023
 ;; Package-Requires: ((emacs "27.1") (org "9.4"))
-;; Keywords: org-mode, annotation, writing, note-taking, marginal notes
+;; Keywords: org-mode, annotation, note-taking, marginal-notes, wp
 
 ;; This file is not part of GNU Emacs.
 
@@ -48,13 +48,31 @@ is an Org file.
 If it is a function, the default function is
 `org-remark-notes-file-name-function'.  It returns a file name
 like this: \"FILE-notes.org\" by adding \"-notes.org\" as a
-suffix to the file name without the extension."
+suffix to the file name without the extension. When the buffer is
+not visiting a file (e.g. a website via EWW), the file name will
+be \"marginalia.org\" in your `user-emacs-directory'."
   :group 'org-remark
   :safe #'stringp
   :type '(choice
           (file "marginalia.org")
           (function org-remark-notes-file-name-function)))
 
+(defcustom org-remark-highlight-link-to-source-functions nil
+  "Abnormal hook called to create a link to source in notes file.
+Each one is called with FILENAME as an argument."
+  :group 'org-remark
+  :type '(repeat function))
+
+(defvar org-remark-source-find-file-name-functions nil
+  "List of functions to get the source file name.
+It is an abnormal hook run with no argument and each function
+must return a file-name-equvalent as a string that uniquely
+identifies the source.  The hook is run when `buffer-file-name`
+in source buffer returns nil, meaning the source buffer is not
+visiting a file.
+
+Meant to be set by extensions such as `org-remark-eww'")
+
 ;;;###autoload
 (define-minor-mode org-remark-global-tracking-mode
   "Automatically activates local minor mode `org-remark-mode'.
@@ -66,14 +84,13 @@ readable, the function automatically activates `org-remark'."
   :lighter " ormk-auto"
   :global t
   :group 'org-remark
-  (cond
-   (org-remark-global-tracking-mode
-    ;; Activate
-    (add-hook 'find-file-hook #'org-remark-auto-on))
-   (t
-    ;; Deactivate
-    (remove-hook 'find-file-hook #'org-remark-auto-on))))
+  (if org-remark-global-tracking-mode
+    ;; Enable
+      (add-hook 'find-file-hook #'org-remark-auto-on)
+    ;; Disable
+    (remove-hook 'find-file-hook #'org-remark-auto-on)))
 
+;;; Functions
 (defun org-remark-notes-file-name-function ()
   "Return a marginal notes file name for the current buffer.
 
@@ -82,14 +99,31 @@ This is the default function for the customizing variable
 
 When the current buffer is visiting a file, the name of marginal
 notes file will be \"FILE-notes.org\", adding \"-notes.org\" as a
-suffix to the file name without the extension."
-  (concat (file-name-sans-extension (buffer-file-name)) "-notes.org"))
+suffix to the file name without the extension.
+
+If the current buffer is not visiting a file, the file name will
+be marginalia.org in your `user-emacs-directory'.  If this file
+name is not suitable, either override the function or set the
+user option to use your own custom function."
+  (if buffer-file-name
+      (let ((source-filename (org-remark-source-find-file-name)))
+        (when (and (stringp source-filename)
+                   (file-exists-p source-filename))
+          (concat (file-name-sans-extension
+                   (file-name-nondirectory source-filename))
+                  "-notes.org")))
+    ;; If buffer is not visiting a file, a default file name.  If this
+    ;; file name is not suitable, either override the function or set
+    ;; the user option to a custom function.
+    (expand-file-name "marginalia.org" user-emacs-directory)))
 
 (defalias
-  'org-remark-notes-file-path-function 'org-remark-notes-file-name-function)
+  'org-remark-notes-file-path-function
+  'org-remark-notes-file-name-function)
 
 (make-obsolete
- 'org-remark-notes-file-path-function 'org-remark-notes-file-name-function "0.2.0" )
+ 'org-remark-notes-file-path-function
+ 'org-remark-notes-file-name-function "0.2.0" )
 
 ;;;; Private Functions
 
@@ -106,8 +140,22 @@ This function is meant to be added to `find-file-hook' by
   "Return the name of marginal notes file for current buffer."
   (if (functionp org-remark-notes-file-name)
       (funcall org-remark-notes-file-name)
-    ;; If not function, assume string and return it as the file path.
-    org-remark-notes-file-name))
+    ;; If not function, assume string and return it as the file name.
+    ;; TODO when buffer is not visitng a file, assume file resides in
+    ;; `user-emacs-directory'
+    (if buffer-file-name org-remark-notes-file-name
+      (expand-file-name org-remark-notes-file-name user-emacs-directory))))
+
+(defun org-remark-source-find-file-name ()
+  "Return the filename for the source buffer.
+We use this filename to identify the source buffer in all
+operations related to marginal notes.
+Assumes that we are currently in the source buffer."
+  (let ((filename (or (and (not (derived-mode-p 'special-mode))
+                           buffer-file-name)
+                      (run-hook-with-args-until-success
+                       'org-remark-source-find-file-name-functions))))
+    filename))
 
 (provide 'org-remark-global-tracking)
 
